@@ -52,8 +52,8 @@
 #include <gst/audio/audio.h>
 #include <string.h>             /* strcmp */
 #include "gstadderorc.h"
-#include <gstlalcollectpads.h>
-#include <gstlal_debug.h>
+#include <gstlal/gstlalcollectpads.h>
+#include <gstlal/gstlal_debug.h>
 
 /* highest positive/lowest negative x-bit value we can use for clamping */
 #define MAX_INT_32  ((gint32) (0x7fffffff))
