@@ -17,6 +17,15 @@
  */
 
 
+/**
+ * SECTION:gstlal_firbank
+ * @short_description:  Projects a single audio channel onto a bank of FIR filters to produce a multi-channel output.
+ *
+ * Reviewed:  3a08854e595cf35a002483eae9a2b001b1100d21 2014-08-14 K.
+ * Cannon, J.  Creighton, B. Sathyaprakash.
+ */
+
+
 /*
  * ============================================================================
  *
@@ -156,7 +165,7 @@ static unsigned fir_length(const GSTLALFIRBank *element)
 static unsigned fft_block_length(const GSTLALFIRBank *element)
 {
 	unsigned block_length = element->block_stride + fir_length(element) - 1;
-	return block_length & 1 ? block_length + 1 : block_length;
+	return block_length + (block_length & 1);
 }
 
 
@@ -270,7 +279,7 @@ static void set_metadata(GSTLALFIRBank *element, GstBuffer *buf, guint64 outsamp
 	GST_BUFFER_OFFSET_END(buf) = element->next_out_offset;
 	GST_BUFFER_TIMESTAMP(buf) = element->t0 + gst_util_uint64_scale_int_round(GST_BUFFER_OFFSET(buf) - element->offset0, GST_SECOND, element->rate);
 	GST_BUFFER_DURATION(buf) = element->t0 + gst_util_uint64_scale_int_round(GST_BUFFER_OFFSET_END(buf) - element->offset0, GST_SECOND, element->rate) - GST_BUFFER_TIMESTAMP(buf);
-	if(element->need_discont) {
+	if(G_UNLIKELY(element->need_discont)) {
 		GST_BUFFER_FLAG_SET(buf, GST_BUFFER_FLAG_DISCONT);
 		element->need_discont = FALSE;
 	}
@@ -490,7 +499,7 @@ static void free_workspace(GSTLALFIRBank *element)
 
 
 /*
- * transform input samples to output samples using a frequency-domain
+ * transform input samples to output samples using a time-domain
  * algorithm.  output_length is the number of output samples to compute.  if
  * this is more than will fit in the output buffer, only as many as will fit in
  * the output buffer will be computed.  the return value is the actual number
@@ -1110,7 +1119,7 @@ static GstFlowReturn do_new_segment(GSTLALFIRBank *element)
 		start = gst_util_uint64_scale_int_round(start, GST_SECOND, element->rate);
 		if(stop != -1) {
 			stop = gst_util_uint64_scale_int_round(stop, element->rate, GST_SECOND);
-			stop += -samples_lost - element->latency;
+			stop += -element->latency;
 			stop = gst_util_uint64_scale_int_round(stop, GST_SECOND, element->rate);
 		}
 		position = start;
@@ -1458,7 +1467,7 @@ static GstFlowReturn transform(GstBaseTransform *trans, GstBuffer *inbuf, GstBuf
 	 */
 
 	g_mutex_lock(element->fir_matrix_lock);
-	while(!element->fir_matrix) {
+	while(G_UNLIKELY(!element->fir_matrix)) {
 		GST_DEBUG_OBJECT(element, "fir matrix not available, waiting ...");
 		g_cond_wait(element->fir_matrix_available, element->fir_matrix_lock);
 		if(GST_STATE(GST_ELEMENT(trans)) == GST_STATE_NULL) {
@@ -1475,7 +1484,7 @@ static GstFlowReturn transform(GstBaseTransform *trans, GstBuffer *inbuf, GstBuf
 	 * size or latency changes, but those cases should produce "update"
 	 * new segment events */
 
-	if(element->need_new_segment) {
+	if(G_UNLIKELY(element->need_new_segment)) {
 		do_new_segment(element);
 		element->need_new_segment = FALSE;
 	}
@@ -1484,7 +1493,7 @@ static GstFlowReturn transform(GstBaseTransform *trans, GstBuffer *inbuf, GstBuf
 	 * check for discontinuity
 	 */
 
-	if(GST_BUFFER_IS_DISCONT(inbuf) || GST_BUFFER_OFFSET(inbuf) != element->next_in_offset || !GST_CLOCK_TIME_IS_VALID(element->t0)) {
+	if(G_UNLIKELY(GST_BUFFER_IS_DISCONT(inbuf) || GST_BUFFER_OFFSET(inbuf) != element->next_in_offset || !GST_CLOCK_TIME_IS_VALID(element->t0))) {
 		/*
 		 * flush any previous history and clear the adapter
 		 */
@@ -1653,7 +1662,7 @@ static void set_property(GObject *object, enum property prop_id, const GValue *v
 		gint block_stride;
 		g_mutex_lock(element->fir_matrix_lock);
 		block_stride = g_value_get_int(value);
-		if(block_stride != element->block_stride && element->time_domain)
+		if(block_stride != element->block_stride && !element->time_domain)
 			/*
 			 * invalidate filter workspace
 			 */
