@@ -20,6 +20,15 @@
  */
 
 
+/**
+ * SECTION:gstlal_gate
+ * @short_description:  Flag buffers as gaps based on the value of a control input.
+ *
+ * Reviewed:  8466e17ed01185bd3182603207d2ac322f502967 2014-08-14 K.
+ * Cannon, J.  Creighton, B. Sathyaprakash.
+ */
+
+
 /*
  * ========================================================================
  *
@@ -172,6 +181,18 @@ static gdouble control_sample_uint32(const gpointer data, guint64 offset)
 }
 
 
+static gdouble control_sample_int64(const gpointer data, guint64 offset)
+{
+	return ABS(((const gint64 *) data)[offset]);
+}
+
+
+static gdouble control_sample_uint64(const gpointer data, guint64 offset)
+{
+	return ((const guint64 *) data)[offset];
+}
+
+
 static gdouble control_sample_float32(const gpointer data, guint64 offset)
 {
 	return fabsf(((const float *) data)[offset]);
@@ -331,8 +352,10 @@ static void control_get_interval(GSTLALGate *element, GstClockTime timestamp, Gs
 				break;
 		} else
 			GST_DEBUG_OBJECT(element, "have 0 control segments");
-		if(element->control_eos)
+		if(element->control_eos) {
+			GST_DEBUG_OBJECT(element, "control is at EOS");
 			break;
+		}
 
 		/*
 		 * no, wait for buffer to arrive
@@ -341,8 +364,6 @@ static void control_get_interval(GSTLALGate *element, GstClockTime timestamp, Gs
 		GST_DEBUG_OBJECT(element, "waiting for control to advance to %" GST_TIME_SECONDS_FORMAT, GST_TIME_SECONDS_ARGS(tmax));
 		g_cond_wait(element->control_queue_head_changed, element->control_lock);
 	}
-	if(element->control_eos)
-		GST_DEBUG_OBJECT(element, "control is at EOS");
 	g_mutex_unlock(element->control_lock);
 }
 
@@ -658,6 +679,9 @@ static gboolean control_setcaps(GstPad *pad, GstCaps *caps)
 		case 32:
 			control_sample_func = is_signed ? control_sample_int32 : control_sample_uint32;
 			break;
+		case 64:
+			control_sample_func = is_signed ? control_sample_int64 : control_sample_uint64;
+			break;
 		default:
 			success = FALSE;
 			break;
@@ -968,7 +992,7 @@ static GstFlowReturn sink_chain(GstPad *pad, GstBuffer *sinkbuf)
 	 * is input zero size or already a gap?  then push it as is
 	 */
 
-	if(!sinkbuf_length) {
+	if(G_UNLIKELY(!sinkbuf_length)) {
 		/*
 		 * is a discontinuity pending?
 		 */
@@ -1383,7 +1407,7 @@ static void gstlal_gate_class_init(GSTLALGateClass *klass)
 			"threshold",
 			"Threshold",
 			"Output will be flagged as non-gap when magnitude of control input is >= this value.  See also invert-control.",
-			0, G_MAXDOUBLE, DEFAULT_THRESHOLD,
+			0, INFINITY, DEFAULT_THRESHOLD,
 			G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT
 		)
 	);
