@@ -1,3 +1,33 @@
+/*
+ * lal_segmentsrc
+ *
+ * Copyright (C) ??
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
+
+/**
+ * SECTION:gstlal_segmentsrc
+ * @short_description:  The output is a buffer of boolean values specifying when a list of segments are on and off.
+ *
+ * Reviewed:  a2d52f933cd71abc2effa66b46d030ee605e7cea 2014-08-13 K.
+ * Cannon, J.  Creighton, B. Sathyaprakash.
+ */
+
+
 /*
  * ============================================================================
  *
@@ -46,8 +76,8 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE(
         "channels = (int) 1, " \
         "endianness = (int) BYTE_ORDER, " \
         "width = (int) 8," \
-        "depth = (int) 8," \
-        "signed = true"
+        "depth = (int) 1," \
+        "signed = false"
     )
 );
 
@@ -64,66 +94,6 @@ enum property {
 };
 
 
-/*
- * ======================================================================
- *
- * utility functions
- *
- * ======================================================================
- */
-
-
-/*
- * return the sample size (this is always 8 for now)
- */
-
-
-static guint sample_size(GstBaseSrc *basesrc)
-{
-    //GSTLALSegmentSrc        *element = GSTLAL_SEGMENTSRC(basesrc);
-    return 1; /* FIXME The only size supported, guint8*/
-}
-
-
-/*
- * round a time to the nearest sample based on sample rate
- */
-
-
-static guint64 round_to_nearest_sample(GstBaseSrc *basesrc, guint64 val)
-{
-    GSTLALSegmentSrc        *element = GSTLAL_SEGMENTSRC(basesrc);
-    return gst_util_uint64_scale_int_round(val, 1, element->rate * GST_SECOND) * element->rate * GST_SECOND;
-}
-
-
-/* FIXME a place holder to somehow infer caps from segments??? maybe useless */
-#if 0
-static GstCaps *segments_to_caps(gint rate)
-{
-    GstCaps *caps;
-
-    /*
-     * FIXME, it would be nice to get say, the rate from the segments...
-     * but that seems hard.  So for now everything is hardcoded
-     * FIXME this function isn't even used yet, we get caps from downstream
-     */
-
-    caps = gst_caps_new_simple(
-        "audio/x-raw-int",
-        "rate", G_TYPE_INT, rate,
-        "channels", G_TYPE_INT, 1,
-        "endianness", G_TYPE_INT, G_BYTE_ORDER,
-        "width", G_TYPE_INT, 8,
-        "depth", G_TYPE_INT, 8,
-        "signed", G_TYPE_BOOLEAN, TRUE,
-        NULL
-    );
-    return caps;
-}
-#endif
-
-
 /*
  * ============================================================================
  *
@@ -133,88 +103,32 @@ static GstCaps *segments_to_caps(gint rate)
  */
 
 
-/*
- * start()
- */
-
-
-static gboolean start(GstBaseSrc *object)
-{
-    return TRUE;
-}
-
-
-/*
- * stop()
- */
-
-
-static gboolean stop(GstBaseSrc *object)
-{
-    return TRUE;
-}
-
-
-/*
- * Mark buffer according to segment
- */
-
-
-static int mark_segment(GstBaseSrc *basesrc, GstBuffer *buffer, guint64 start, guint64 stop)
-{
-    GSTLALSegmentSrc *element = GSTLAL_SEGMENTSRC(basesrc);
-    guint startix=0;
-    guint stopix =0;
-    gint8 *data = NULL;
-
-    if (start > GST_BUFFER_TIMESTAMP(buffer))
-        startix = round_to_nearest_sample(basesrc, start - GST_BUFFER_TIMESTAMP(buffer)) / element->rate / GST_SECOND;
-    else
-        startix = 0;
-
-    if (stop > GST_BUFFER_TIMESTAMP(buffer))
-        stopix = round_to_nearest_sample(basesrc, stop - GST_BUFFER_TIMESTAMP(buffer)) / element->rate / GST_SECOND;
-    else
-        stopix = 0;
-
-    data = (gint8 *) GST_BUFFER_DATA(buffer);
-
-    if (element->invert_output)
-        for (guint32 i = startix; i < stopix; i++) data[i] = 0;
-    else
-        for (guint32 i = startix; i < stopix; i++) data[i] = G_MAXINT8;
-
-    return 0;
-}
-
-
 /*
  * Mark buffer according to segment list
  */
 
 
-static int mark_segments(GstBaseSrc *basesrc, GstBuffer *buffer, guint64 start, guint64 stop)
+static int mark_segments(GSTLALSegmentSrc *element, GstBuffer *buffer)
 {
-    GSTLALSegmentSrc        *element = GSTLAL_SEGMENTSRC(basesrc);
-    struct gstlal_segment_list *seglist = element->seglist;
-    if (!seglist) return 0; /* FIXME handle no segment lists */
-    guint rows = seglist->length;
-    guint64 segstart, segstop;
+    guint8 *data = GST_BUFFER_DATA(buffer);
+    GstClockTime start = GST_BUFFER_TIMESTAMP(buffer);
+    GstClockTime stop = GST_BUFFER_TIMESTAMP(buffer) + GST_BUFFER_DURATION(buffer);
+    gint i;
 
-    /* FIXME provide a bailout and a sensible starting point if you have sorted and coalesced segents */
-    /* This is ridiculous, but doesn't require sorted or coalesced segments.  Could some fancy data structure help? */
-    /* FIXME switch to using gstlal segment routines */
-    for (guint i = 0; i < rows; i++) {
-        segstart = seglist->segments[i].start;
-        segstop = seglist->segments[i].stop;
-        if ((segstart >= start) && (segstart < stop) && (segstop < stop))
-            mark_segment(basesrc, buffer, segstart, segstop);
-        if ((segstart >= start) && (segstart < stop) && (segstop >= stop))
-            mark_segment(basesrc, buffer, segstart, stop);
-        if ((segstop >= start) && (segstop < stop) && (segstart < start))
-            mark_segment(basesrc, buffer, start, segstop);
-        if ((segstart <= start) && (segstop >= stop))
-            mark_segment(basesrc, buffer, start, stop);
+    /* This is ridiculous, but doesn't require sorted or coalesced
+     * segments.  Could some fancy data structure help? */
+    for (i = 0; i < element->seglist->length; i++) {
+    	/* clip segment to buffer */
+        GstClockTime segstart = CLAMP(element->seglist->segments[i].start, start, stop);
+        GstClockTime segstop  = CLAMP(element->seglist->segments[i].stop,  start, stop);
+
+	/* convert to samples */
+	guint64 startix = gst_util_uint64_scale_int_round(segstart - start, element->rate, GST_SECOND);
+	guint64 stopix  = gst_util_uint64_scale_int_round(segstop  - start, element->rate, GST_SECOND);
+
+	/* set samples */
+	for(; startix < stopix; startix++)
+		data[startix] = element->invert_output ? 0 : 0x80;
     }
 
     return 0;
@@ -228,58 +142,45 @@ static int mark_segments(GstBaseSrc *basesrc, GstBuffer *buffer, guint64 start,
 
 static GstFlowReturn create(GstBaseSrc *basesrc, guint64 offset, guint size, GstBuffer **buffer)
 {
-    GSTLALSegmentSrc        *element = GSTLAL_SEGMENTSRC(basesrc);
-    GstFlowReturn result;
+    GSTLALSegmentSrc *element = GSTLAL_SEGMENTSRC(basesrc);
+    GstFlowReturn result = GST_FLOW_OK;
     gulong blocksize = gst_base_src_get_blocksize(basesrc);
-    guint samplesize = sample_size(basesrc);
-    guint64 numsamps = blocksize / samplesize;
-    guint64 start, stop;
+    guint64 numsamps = blocksize;
+    GstClockTime start = basesrc->segment.start + gst_util_uint64_scale_int_round(basesrc->offset, GST_SECOND, element->rate);
+    GstClockTime stop = basesrc->segment.start + gst_util_uint64_scale_int_round(basesrc->offset + numsamps, GST_SECOND, element->rate);
+
+    *buffer = NULL;	/* just in case */
 
     /*
-     * Bail if the requested block size doesn't correspond to integer samples
+     * Check for EOS
      */
 
-    if (blocksize % samplesize) {
-        GST_ERROR_OBJECT(element,
-            "block size %lu is not an integer multiple of the sample size %lu",
-            blocksize, samplesize);
-        return GST_FLOW_ERROR;
-    }
+    if(GST_CLOCK_TIME_IS_VALID(basesrc->segment.stop) && start >= (GstClockTime) basesrc->segment.stop)
+        return GST_FLOW_UNEXPECTED;
 
     /*
      * Allocate the buffer of ones or zeros depending on the invert property
      */
 
     result = gst_pad_alloc_buffer(GST_BASE_SRC_PAD(basesrc), basesrc->offset, blocksize, GST_PAD_CAPS(GST_BASE_SRC_PAD(basesrc)), buffer);
+    if(result != GST_FLOW_OK)
+        return result;
 
-    gint8 *d = (gint8 *) GST_BUFFER_DATA(*buffer);
-    if (element->invert_output)
-        for (guint32 i = 0; i < numsamps; i++) d[i] = G_MAXINT8;
-    else {
-        for (guint32 i = 0; i < numsamps; i++) d[i] = 0;
-    }
+    memset(GST_BUFFER_DATA(*buffer), element->invert_output ? 0x80 : 0x00, GST_BUFFER_SIZE(*buffer));
 
     /*
      * update the offsets, timestamps etc
      */
 
     GST_BUFFER_OFFSET_END(*buffer) = GST_BUFFER_OFFSET(*buffer) + numsamps;
-
-    start = GST_BUFFER_TIMESTAMP(*buffer) = basesrc->segment.start
-        + gst_util_uint64_scale_int_round(GST_BUFFER_OFFSET(*buffer), GST_SECOND, element->rate);
-
-    stop = basesrc->segment.start
-        + gst_util_uint64_scale_int_round(GST_BUFFER_OFFSET_END(*buffer), GST_SECOND, element->rate);
-
+    GST_BUFFER_TIMESTAMP(*buffer) = start;
     GST_BUFFER_DURATION(*buffer) = stop - start;
-    
+
     /*
      * Mark the buffer according to the segments
      */
 
-    mark_segments(basesrc, *buffer, start, stop);
-
-    /* FIXME Huh? */
+    mark_segments(element, *buffer);
     if(basesrc->offset == 0)
         GST_BUFFER_FLAG_SET(*buffer, GST_BUFFER_FLAG_DISCONT);
 
@@ -308,7 +209,6 @@ static gboolean is_seekable(GstBaseSrc *object)
 static gboolean do_seek(GstBaseSrc *basesrc, GstSegment *segment)
 {
     GSTLALSegmentSrc        *element = GSTLAL_SEGMENTSRC(basesrc);
-    GstClockTime epoch;
 
     /*
      * Parse the segment
@@ -318,7 +218,6 @@ static gboolean do_seek(GstBaseSrc *basesrc, GstSegment *segment)
         GST_ELEMENT_ERROR(element, RESOURCE, SEEK, ("seek failed:  start time is required"), (NULL));
         return FALSE;
     }
-    epoch = segment->start;
 
     /*
      * Try doing the seek
@@ -372,11 +271,13 @@ static gboolean query(GstBaseSrc *basesrc, GstQuery *query)
 					break;
 
 				case GST_FORMAT_BYTES:
-					offset = src_value / (element->width / 8);
+					/* width is 8 bits */
+					offset = src_value;
 					break;
 
 				case GST_FORMAT_BUFFERS:
-					offset = gst_util_uint64_scale_int_round(src_value, gst_base_src_get_blocksize(basesrc), element->width / 8);
+					/* width is 8 bits */
+					offset = src_value * gst_base_src_get_blocksize(basesrc);
 					break;
 
 				case GST_FORMAT_PERCENT:
@@ -403,11 +304,13 @@ static gboolean query(GstBaseSrc *basesrc, GstQuery *query)
 					break;
 
 				case GST_FORMAT_BYTES:
-					dest_value = offset * (element->width / 8);
+					/* width is 8 bits */
+					dest_value = offset;
 					break;
 
 				case GST_FORMAT_BUFFERS:
-					dest_value = gst_util_uint64_scale_int_ceil(offset, element->width / 8, gst_base_src_get_blocksize(basesrc));
+					/* width is 8 bits */
+					dest_value = gst_util_uint64_scale_int_ceil(offset, 1, gst_base_src_get_blocksize(basesrc));
 					break;
 
 				case GST_FORMAT_PERCENT:
@@ -448,25 +351,6 @@ static gboolean check_get_range(GstBaseSrc *basesrc)
  */
 
 
-/*
- * Compare function to sort segment list
- */
-
-
-gint seg_compare_func(gconstpointer a, gconstpointer b)
-{
-    GValueArray *rowa = (GValueArray *) g_value_get_boxed((GValue *) a);
-    GValueArray *rowb = (GValueArray *) g_value_get_boxed((GValue *) b);
-    guint64 astart = g_value_get_uint64(g_value_array_get_nth(rowa, 0));
-    guint64 bstart = g_value_get_uint64(g_value_array_get_nth(rowb, 0));
-
-    if (astart <  bstart) return -1;
-    if (astart == bstart) return 0;
-    if (astart >  bstart) return 1;
-    return 0;
-}
-
-
 /*
  * set_property()
  */
@@ -480,7 +364,10 @@ static void set_property(GObject *object, enum property prop_id, const GValue *v
 
     switch (prop_id) {
         case ARG_SEGMENT_LIST:
+            g_mutex_lock(element->segment_matrix_lock);
+            gstlal_segment_list_free(element->seglist);
             element->seglist = gstlal_segment_list_from_g_value_array(g_value_get_boxed(value));
+            g_mutex_unlock(element->segment_matrix_lock);
             break;
         case ARG_INVERT_OUTPUT:
             element->invert_output = g_value_get_boolean(value);
@@ -534,14 +421,16 @@ static void get_property(GObject *object, enum property prop_id, GValue *value,
 
 static void finalize(GObject *object)
 {
-    GSTLALSegmentSrc        *element = GSTLAL_SEGMENTSRC(object);
+    GSTLALSegmentSrc *element = GSTLAL_SEGMENTSRC(object);
 
     /*
      * free resources
      */
 
-    if (element->seglist) gstlal_segment_list_free(element->seglist);
+    gstlal_segment_list_free(element->seglist);
+    element->seglist = NULL;
     g_mutex_free(element->segment_matrix_lock);
+    element->segment_matrix_lock = NULL;
 
     /*
      * chain to parent class' finalize() method
@@ -571,11 +460,8 @@ static gboolean set_caps(GstBaseSrc *src, GstCaps *caps)
         success = FALSE;
     }
 
-    if(success) {
-        if(rate != element->rate)
-            GST_DEBUG_OBJECT(element, "rate changed, but no signal was emitted because it is not implmented: %d -> %d ", element->rate, rate);
+    if(success)
         element->rate = rate;
-    }
 
     return success;
 }
@@ -587,10 +473,6 @@ static gboolean set_caps(GstBaseSrc *src, GstCaps *caps)
 
 static void gstlal_segmentsrc_base_init(gpointer gclass)
 {
-    GstElementClass     *element_class = GST_ELEMENT_CLASS(gclass);
-
-    gst_element_class_set_details_simple(element_class, "List of on times and off times", "Source/Audio", "The output is a buffer of boolean values specifying when a list of segments are on and off.", "Collin Capano <collin.capano@ligo.org>");
-    gst_element_class_add_pad_template(element_class, gst_static_pad_template_get(&src_factory));
 }
 
 
@@ -600,12 +482,18 @@ static void gstlal_segmentsrc_base_init(gpointer gclass)
 
 static void gstlal_segmentsrc_class_init(GSTLALSegmentSrcClass *klass)
 {
-    GObjectClass        *gobject_class = G_OBJECT_CLASS(klass);
     GstBaseSrcClass *gstbasesrc_class = GST_BASE_SRC_CLASS(klass);
+    GstElementClass *element_class = GST_ELEMENT_CLASS(klass);
+    GObjectClass    *gobject_class = G_OBJECT_CLASS(klass);
+
+    gst_element_class_set_details_simple(element_class, "List of on times and off times", "Source/Audio", "The output is a buffer of boolean values specifying when a list of segments are on and off.", "Collin Capano <collin.capano@ligo.org>");
+
     gobject_class->set_property = GST_DEBUG_FUNCPTR(set_property);
     gobject_class->get_property = GST_DEBUG_FUNCPTR(get_property);
     gobject_class->finalize = GST_DEBUG_FUNCPTR(finalize);
 
+    gst_element_class_add_pad_template(element_class, gst_static_pad_template_get(&src_factory));
+
     g_object_class_install_property(
         gobject_class,
         ARG_SEGMENT_LIST,
@@ -646,8 +534,6 @@ static void gstlal_segmentsrc_class_init(GSTLALSegmentSrcClass *klass)
      * GstBaseSrc method overrides
      */
 
-    gstbasesrc_class->start = GST_DEBUG_FUNCPTR(start);
-    gstbasesrc_class->stop = GST_DEBUG_FUNCPTR(stop);
     gstbasesrc_class->create = GST_DEBUG_FUNCPTR(create);
     gstbasesrc_class->is_seekable = GST_DEBUG_FUNCPTR(is_seekable);
     gstbasesrc_class->do_seek = GST_DEBUG_FUNCPTR(do_seek);
@@ -665,8 +551,6 @@ static void gstlal_segmentsrc_init(GSTLALSegmentSrc *segment_src, GSTLALSegmentS
 {
     segment_src->seglist = NULL;
     segment_src->rate = 0;
-    /* FIXME hardcoded width */
-    segment_src->width = 8;
     segment_src->segment_matrix_lock = g_mutex_new();
     gst_base_src_set_format(GST_BASE_SRC(segment_src), GST_FORMAT_TIME);
 }
