@@ -276,15 +276,21 @@ static int sim_inspiral_strain(REAL8TimeSeries **strain, SimInspiralTable *sim_i
 	REAL8TimeSeries *hcross = NULL;
 
 	/*
-	 * create waveform using lalsimulation
+	 * create waveform using lalinspiral's wrapping of lalsimulation.
+	 * XLALInspiralTDWaveformFromSimInspiral() translates the
+	 * sim_inspiral row into a function call into lalsimulation,
+	 * collecting and returning a conditioned waveform suitable for
+	 * injection into an h(t) stream (after projection onto an antenna
+	 * respose).
 	 *
-	 * FIXME:  remove locks when we can be sure lal has been compiled with pthread support
+	 * FIXME:  remove locks when we can be sure lal has been compiled
+	 * with pthread support
 	 */
 
 #ifndef LAL_PTHREAD_LOCK
 	gstlal_fftw_lock();
 #endif
-	if(XLALSimInspiralChooseWaveformFromSimInspiral(&hplus, &hcross, sim_inspiral, deltaT) == XLAL_FAILURE) {
+	if(XLALInspiralTDWaveformFromSimInspiral(&hplus, &hcross, sim_inspiral, deltaT) == XLAL_FAILURE) {
 #ifndef LAL_PTHREAD_LOCK
 		gstlal_fftw_unlock();
 #endif
@@ -317,7 +323,6 @@ static int sim_inspiral_strain(REAL8TimeSeries **strain, SimInspiralTable *sim_i
 
 static int update_simulation_series(REAL8TimeSeries *h, GSTLALSimulation *element, const COMPLEX16FrequencySeries *response)
 {
-	int twice_pn_order = 7;
 	double injTime, DeltaT, startMinusEnd, endMinusStart;
 	REAL8 tmpREAL8;
 	LIGOTimeGPS hStartTime, hEndTime, injStartTime, injEndTime;
@@ -379,17 +384,21 @@ static int update_simulation_series(REAL8TimeSeries *h, GSTLALSimulation *elemen
 	 * loop over injections in file
 	 */
 
-	thisSimInspiral = element->injection_document->sim_inspiral_table_head;
-	while(thisSimInspiral) {
+	for(thisSimInspiral = element->injection_document->sim_inspiral_table_head; thisSimInspiral; ) {
+		REAL8TimeSeries *inspiral_series = NULL;
 
 		/*
 		 * calculate start and end times for this series containing
-		 * this injection
+		 * this injection.  NOTE:  this just uses the 0 PN formula
+		 * and pads it to be safe
 		 */
 
-		injTime = XLALSimInspiralTaylorLength(1.0 / 4096, thisSimInspiral->mass1 * LAL_MSUN_SI, thisSimInspiral->mass2 * LAL_MSUN_SI, thisSimInspiral->f_lower, twice_pn_order);
+		/* the padding will no longer be sufficient at this mass */
+		g_assert((thisSimInspiral->mass1 + thisSimInspiral->mass2) < 1e4);
+
+		injTime = 1.0 + XLALSimInspiralTaylorF2ReducedSpinChirpTime(thisSimInspiral->f_lower, thisSimInspiral->mass1 * LAL_MSUN_SI, thisSimInspiral->mass2 * LAL_MSUN_SI, 0, 0);
 		injStartTime = injEndTime = thisSimInspiral->geocent_end_time;
-		XLALGPSAdd(&injStartTime, -(1.9*injTime - 1.0));
+		XLALGPSAdd(&injStartTime, -1.9*injTime - 1.0);
 		XLALGPSAdd(&injEndTime, 0.1*injTime + 1.0);
 
 		/*
@@ -420,23 +429,22 @@ static int update_simulation_series(REAL8TimeSeries *h, GSTLALSimulation *elemen
 		 * compute injection waveform
 		 */
 
-		REAL8TimeSeries *inspiral_series = NULL;
 		if(sim_inspiral_strain(&inspiral_series, thisSimInspiral, h->deltaT, *detector))
 			XLAL_ERROR(XLAL_EFUNC);
 
-		// FIXME: should we check that the start of the waveform is contained in simulation_series?
-
 		/*
 		 * resize simulation_series to cover this time
 		 */
 
-		DeltaT = XLALGPSDiff(&element->simulation_series->epoch, &inspiral_series->epoch);
-		DeltaT += element->simulation_series->deltaT * element->simulation_series->data->length;
-		DeltaT -= inspiral_series->deltaT * inspiral_series->data->length;
-		if(DeltaT < 1.) 
-			element->simulation_series = XLALResizeREAL8TimeSeries(element->simulation_series, 0, element->simulation_series->data->length + ceil((1. - DeltaT) / element->simulation_series->deltaT));
-		if(!element->simulation_series)
+		DeltaT = XLALGPSDiff(&inspiral_series->epoch, &element->simulation_series->epoch);
+		DeltaT += inspiral_series->data->length * inspiral_series->deltaT + injection_window;
+		DeltaT -= element->simulation_series->data->length * element->simulation_series->deltaT;
+		if(DeltaT > 0.)
+			element->simulation_series = XLALResizeREAL8TimeSeries(element->simulation_series, 0, element->simulation_series->data->length + ceil(DeltaT / element->simulation_series->deltaT));
+		if(!element->simulation_series) {
+			XLALDestroyREAL8TimeSeries(inspiral_series);
 			XLAL_ERROR(XLAL_EFUNC);
+		}
 
 		/*
 		 * add detector strain to simulation_series
@@ -467,11 +475,11 @@ static int update_simulation_series(REAL8TimeSeries *h, GSTLALSimulation *elemen
 		if(prevSimInspiral)
 			prevSimInspiral->next = thisSimInspiral->next;
 		{
-			SimInspiralTable *tmpSimInspiral = thisSimInspiral;
-			thisSimInspiral = thisSimInspiral->next;
-			if (tmpSimInspiral == element->injection_document->sim_inspiral_table_head)
-				element->injection_document->sim_inspiral_table_head = thisSimInspiral;
-			XLALFree(tmpSimInspiral);
+		SimInspiralTable *tmpSimInspiral = thisSimInspiral;
+		thisSimInspiral = thisSimInspiral->next;
+		if (tmpSimInspiral == element->injection_document->sim_inspiral_table_head)
+			element->injection_document->sim_inspiral_table_head = thisSimInspiral;
+		XLALFree(tmpSimInspiral);
 		}
 	}
 
@@ -667,13 +675,21 @@ static GstFlowReturn transform_ip(GstBaseTransform *trans, GstBuffer *buf)
 		/* FIXME:  hard-coded = BAD BAD BAD */
 		/*XLALINT8NSToGPS(&start, (INT8) 1 << 63);
 		XLALINT8NSToGPS(&end, ((INT8) 1 << 63) - 1);*/
-		/* FIXME we don't understand what about this funciton isn't
-		 * thread safe, so we need to figure that out properly and
-		 * remove the locks.
-		 */
+		/* FIXME remove locks when we can be sure LAL has been
+		 * compiled with pthread support.
+		 * load_injection_document() relies on testing the xlal
+		 * errno variable, which is only thread-safe when lal was
+		 * compiled with pthread locking turned on.  previously
+		 * there have also been thread safety problems in
+		 * libmetaio, but those have been fixed so it's just lal
+		 * that we're waiting on now. */
+#ifndef LAL_PTHREAD_LOCK
 		gstlal_fftw_lock();
+#endif
 		element->injection_document = load_injection_document(element->xml_location, start, end, 0.0);
+#ifndef LAL_PTHREAD_LOCK
 		gstlal_fftw_unlock();
+#endif
 		if(!element->injection_document) {
 			GST_ELEMENT_ERROR(element, RESOURCE, READ, (NULL), ("error loading \"%s\"", element->xml_location));
 			result = GST_FLOW_ERROR;
@@ -682,7 +698,7 @@ static GstFlowReturn transform_ip(GstBaseTransform *trans, GstBuffer *buf)
 	}
 
 	/*
-	 * If stream tags not sufficient, reduce to pass-through
+	 * Make sure stream tags are sufficient
 	 */
 
 	if(!element->instrument || !element->channel_name || !element->units) {
