@@ -17,6 +17,15 @@
  */
 
 
+/**
+ * SECTION:gstlal_statevector
+ * @short_description:  Converts a state vector stream into booleans, for example to drive a lal_gate element.
+ *
+ * Reviewed:  f989b34f43aec056f021f10e5e01866846a3c58d 2014-08-10 K.
+ * Cannon, J.  Creighton, B. Sathyaprakash.
+ */
+
+
 /*
  * ============================================================================
  *
@@ -72,6 +81,12 @@ GST_DEBUG_CATEGORY_STATIC(GST_CAT_DEFAULT);
  */
 
 
+static gboolean required_on_off_too_wide(GSTLALStateVector *element)
+{
+	return (element->required_on | element->required_off) & ~element->mask ? TRUE : FALSE;
+}
+
+
 static guint get_input_uint8(void **in)
 {
 	return *(*(guint8 **) in)++;
@@ -202,6 +217,9 @@ static GstCaps *transform_caps(GstBaseTransform *trans, GstPadDirection directio
 	GstCaps *othercaps = NULL;
 	guint i;
 
+	if(gst_caps_get_size(caps) > 1)
+		GST_WARNING_OBJECT(trans, "not yet smart enough to transform complex formats");
+
 	switch(direction) {
 	case GST_PAD_SRC:
 		/*
@@ -219,16 +237,12 @@ static GstCaps *transform_caps(GstBaseTransform *trans, GstPadDirection directio
 
 	case GST_PAD_SINK:
 		/*
-		 * source pad must have same sample rate as sink pad
-		 * FIXME:  this doesn't work out all the allowed
-		 * permutations, it just takes the rate from the
-		 * first structure on the sink pad and copies it into all
-		 * the structures on the source pad
+		 * source pad's format must be 8-bit boolean
 		 */
 
-		othercaps = gst_caps_copy(gst_pad_get_pad_template_caps(GST_BASE_TRANSFORM_SRC_PAD(trans)));
-		for(i = 0; i < gst_caps_get_size(caps); i++)
-			gst_structure_set_value(gst_caps_get_structure(othercaps, i), "rate", gst_structure_get_value(gst_caps_get_structure(caps, 0), "rate"));
+		othercaps = gst_caps_copy(caps);
+		for(i = 0; i < gst_caps_get_size(othercaps); i++)
+			gst_structure_set(gst_caps_get_structure(othercaps, i), "width", G_TYPE_INT, 8, "depth", G_TYPE_INT, 1, "signed", G_TYPE_BOOLEAN, FALSE, NULL);
 		break;
 
 	case GST_PAD_UNKNOWN:
@@ -285,6 +299,9 @@ static gboolean set_caps(GstBaseTransform *trans, GstCaps *incaps, GstCaps *outc
 		break;
 	}
 
+	if(required_on_off_too_wide(element))
+		GST_WARNING_OBJECT(element, "required-on and/or required-off too wide for stream format;  high-order bits will be ignored");
+
 	if(!success)
 		GST_ERROR_OBJECT(element, "unable to parse and/or accept caps %" GST_PTR_FORMAT, incaps);
 
@@ -316,6 +333,9 @@ static GstFlowReturn transform(GstBaseTransform *trans, GstBuffer *inbuf, GstBuf
 {
 	GSTLALStateVector *element = GSTLAL_STATEVECTOR(trans);
 	GstFlowReturn result = GST_FLOW_OK;
+	guint64 on_samples = element->on_samples;
+	guint64 off_samples = element->off_samples;
+	guint64 gap_samples = element->gap_samples;
 
 	g_assert(element->get_input != NULL);
 
@@ -352,6 +372,17 @@ static GstFlowReturn transform(GstBaseTransform *trans, GstBuffer *inbuf, GstBuf
 		element->gap_samples += GST_BUFFER_OFFSET_END(inbuf) - GST_BUFFER_OFFSET(inbuf);
 	}
 
+	/*
+	 * notify sample count changes
+	 */
+
+	if(element->on_samples != on_samples)
+		g_object_notify(G_OBJECT(trans), "on-samples");
+	if(element->off_samples != off_samples)
+		g_object_notify(G_OBJECT(trans), "off-samples");
+	if(element->gap_samples != gap_samples)
+		g_object_notify(G_OBJECT(trans), "gap-samples");
+
 	/*
 	 * done
 	 */
@@ -403,6 +434,9 @@ static void set_property(GObject *object, enum property prop_id, const GValue *v
 		break;
 	}
 
+	if(required_on_off_too_wide(element))
+		GST_WARNING_OBJECT(element, "required-on and/or required-off too wide for stream format;  high-order bits will be ignored");
+
 	GST_OBJECT_UNLOCK(element);
 }
 
@@ -507,7 +541,7 @@ static void gstlal_statevector_class_init(GSTLALStateVectorClass *klass)
 			"on-samples",
 			"On samples",
 			"Number of samples seen thus far marked as on",
-			0, G_MAXUINT, 0,
+			0, G_MAXUINT64, 0,
 			G_PARAM_READABLE | G_PARAM_STATIC_STRINGS
 		)
 	);
@@ -518,7 +552,7 @@ static void gstlal_statevector_class_init(GSTLALStateVectorClass *klass)
 			"off-samples",
 			"Off samples",
 			"Number of samples seen thus far marked as off",
-			0, G_MAXUINT, 0,
+			0, G_MAXUINT64, 0,
 			G_PARAM_READABLE | G_PARAM_STATIC_STRINGS
 		)
 	);
@@ -529,7 +563,7 @@ static void gstlal_statevector_class_init(GSTLALStateVectorClass *klass)
 			"gap-samples",
 			"Gap samples",
 			"number of samples seen thus far marked as gap",
-			0, G_MAXUINT, 0,
+			0, G_MAXUINT64, 0,
 			G_PARAM_READABLE | G_PARAM_STATIC_STRINGS
 		)
 	);
@@ -546,4 +580,7 @@ static void gstlal_statevector_init(GSTLALStateVector *filter, GSTLALStateVector
 	gst_base_transform_set_gap_aware(GST_BASE_TRANSFORM(filter), TRUE);
 
 	filter->get_input = NULL;
+	/* this value makes all required_on/off masks appear to be OK until
+	 * we know otherwise */
+	filter->mask = 0xffffffff;
 }
