@@ -17,6 +17,15 @@
  */
 
 
+/**
+ * SECTION:gstlal_togglecomplex
+ * @short_description:  Toggle complex-valued <--> real-valued format.
+ *
+ * Reviewed:  8d9a33803cbb87f0844001a2207c5e2e55c9340c 2014-08-10 K.
+ * Cannon, J.  Creighton, B. Sathyaprakash.
+ */
+
+
 /*
  * ============================================================================
  *
@@ -303,8 +312,18 @@ static GstFlowReturn prepare_output_buffer(GstBaseTransform *trans, GstBuffer *i
 
 static void gstlal_togglecomplex_base_init(gpointer gclass)
 {
-	GstElementClass *element_class = GST_ELEMENT_CLASS(gclass);
-	GstBaseTransformClass *transform_class = GST_BASE_TRANSFORM_CLASS(gclass);
+}
+
+
+/*
+ * class_init()
+ */
+
+
+static void gstlal_togglecomplex_class_init(GSTLALToggleComplexClass *klass)
+{
+	GstElementClass *element_class = GST_ELEMENT_CLASS(klass);
+	GstBaseTransformClass *transform_class = GST_BASE_TRANSFORM_CLASS(klass);
 
 	gst_element_class_set_details_simple(
 		element_class,
@@ -323,16 +342,6 @@ static void gstlal_togglecomplex_base_init(gpointer gclass)
 }
 
 
-/*
- * class_init()
- */
-
-
-static void gstlal_togglecomplex_class_init(GSTLALToggleComplexClass *klass)
-{
-}
-
-
 /*
  * init()
  */
