@@ -1,7 +1,7 @@
 /*
  * PSD Estimation and whitener
  *
- * Copyright (C) 2008-2011  Kipp Cannon, Chad Hanna, Drew Keppel
+ * Copyright (C) 2008-2014  Kipp Cannon, Chad Hanna, Drew Keppel
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,44 @@
  */
 
 
+/**
+ * SECTION:gstlal_whiten
+ * @short_description:  Whiten coloured Gaussian noise.
+ *
+ * This element constructs a whitening filter from a power spectral density
+ * function, which it uses to transform coloured Gaussian noise into white
+ * Gaussian noise.  The power spectral density function can be provided
+ * externally or the element can measure it from the time series data using
+ * the <ulink
+ * url="https://www.lsc-group.phys.uwm.edu/daswg/projects/lal/nightly/docs/html/group___time_freq_f_f_t__h.html#ga155912059da0e88b76856c3a72800223">LALPSDRegressor</ulink>
+ * machinery.  When measuring the spectrum internally, the element will
+ * track changes in the spectrum.  The #GSTLALWhiten:average-samples and
+ * #GSTLALWhiten:n-samples parameters control the time scales of the
+ * averaging.
+ *
+ * The power spectral density function measured by the element is available
+ * to external code via three communication channels.  The PSD and its
+ * physical parameters can be retrieved through the #GSTLALWhiten:delta-f,
+ * #GSTLALWhiten:f-nyquist, and #GSTLALWhiten:psd properites.  Use
+ * GObject's notify mechanism to be informed of changes in these values.
+ * When a new PSD is measured, the element posts a #GstMessage to the
+ * pipeline's message bus containing the PSD and its properites (see
+ * #gstlal_whiten_message_psd_parse() for parsing the message).  Finally, a
+ * request pad named "mean-psd" is available.  If the pad is present,
+ * each PSD is packed into a #GstBuffer and pushed out that #GstPad.
+ *
+ * To set a PSD, the #GSTLALWhiten:f-nyquist property should be consulted.
+ * This property gives the Nyquist frequency (in Hertz) of the PSD data
+ * that must be provided and is determined by the sample rate of the time
+ * series.  The #GSTLALWhiten:fft-length property sets the length of the
+ * FFT blocks that will be used for filtering the data, and thus determines
+ * the value of the #GSTLALWhiten:delta-f property.
+ * 
+ * Reviewed:  cf799de0ed55bf454fcd7fd8bc88dcaa8c4bd97e 2014-08-10 K. Cannon, J.
+ * Creighton, B. Sathyaprakash.
+ */
+
+
 /*
  * ========================================================================
  *
@@ -422,8 +460,8 @@ static REAL8FrequencySeries *make_empty_psd(double f0, double deltaF, int length
 static REAL8FrequencySeries *make_unit_psd(double f0, double deltaF, int length, LALUnit sample_units)
 {
 	REAL8FrequencySeries *psd = make_empty_psd(f0, deltaF, length, sample_units);
-	double f_nyquist = f0 + length + deltaF;
-	int n = round(f_nyquist / deltaF) + 1;
+	double f_nyquist = f0 + (length - 1) * deltaF;
+	int n = round(f_nyquist / deltaF);
 	unsigned i;
 
 	if(psd)
@@ -432,6 +470,16 @@ static REAL8FrequencySeries *make_unit_psd(double f0, double deltaF, int length,
 	for(i = 0; i < psd->data->length; i++)
 		psd->data->data[i] = 1 / (n * deltaF);
 
+	/*
+	 * the DC and Nyquist bins have 1/2 the magnitude.  we assume the
+	 * time-domain data had an even number of samples so that the last
+	 * bin is the Nyquist component
+	 */
+
+	if(f0 == 0.)
+		psd->data->data[0] /= 2.;
+	psd->data->data[psd->data->length - 1] /= 2.
+
 	return psd;
 }
 #endif
@@ -532,10 +580,10 @@ static REAL8FrequencySeries *get_psd(GSTLALWhiten *element)
  */
 
 
-static GstMessage *psd_message_new(GSTLALWhiten *element, REAL8FrequencySeries *psd)
+GstMessage *gstlal_whiten_message_psd_new(GSTLALWhiten *element, const gchar *instrument, const REAL8FrequencySeries *psd)
 {
 	GValueArray *va = gstlal_g_value_array_from_doubles(psd->data->data, psd->data->length);
-	char units[50];
+	char units[LALUnitTextSize];
 	GstStructure *s = gst_structure_new(
 		"spectrum",
 		"delta-f", G_TYPE_DOUBLE, psd->deltaF,
@@ -546,12 +594,49 @@ static GstMessage *psd_message_new(GSTLALWhiten *element, REAL8FrequencySeries *
 	GstMessage *m = gst_message_new_element(GST_OBJECT(element), s);
 	g_value_array_free(va);
 
+	if(instrument)
+		gst_structure_set(s, "instrument", G_TYPE_STRING, instrument, NULL);
+
 	GST_MESSAGE_TIMESTAMP(m) = XLALGPSToINT8NS(&psd->epoch);
 
 	return m;
 }
 
 
+REAL8FrequencySeries *gstlal_whiten_message_psd_parse(GstMessage *m)
+{
+	const GstStructure *s = gst_message_get_structure(m);
+	LIGOTimeGPS epoch;
+	LALUnit sample_units;
+	double delta_f;
+	gint length;
+	REAL8FrequencySeries *psd;
+
+	if(!(gst_structure_has_name(s, "spectrum") && gst_structure_get_double(s, "delta-f", &delta_f) && gst_structure_has_field(s, "sample-units") && gst_structure_has_field(s, "magnitude"))) {
+		GST_ERROR("invalid spectrum message: %" GST_PTR_FORMAT, m);
+		return NULL;
+	}
+
+	XLALINT8NSToGPS(&epoch, GST_MESSAGE_TIMESTAMP(m));
+	if(!XLALParseUnitString(&sample_units, gst_structure_get_string(s, "sample-units"))) {
+		GST_ERROR("XLALParseUnitString() failed on sample-units: %" GST_PTR_FORMAT, m);
+		return NULL;
+	}
+
+	psd = XLALCreateREAL8FrequencySeries(gst_structure_get_string(s, "instrument"), &epoch, 0.0, delta_f, &sample_units, 0);
+	if(!psd) {
+		GST_ERROR("XLALCreateREAL8FrequencySeries() failed");
+		return NULL;
+	}
+
+	XLALFree(psd->data->data);
+	psd->data->data = gstlal_doubles_from_g_value_array(g_value_get_boxed(gst_structure_get_value(s, "magnitude")), NULL, &length);
+	psd->data->length = length;
+
+	return psd;
+}
+
+
 static GstFlowReturn push_psd(GstPad *psd_pad, const REAL8FrequencySeries *psd, gint period_N, gint period_D)
 {
 	GstBuffer *buffer = NULL;
@@ -714,7 +799,7 @@ static GstFlowReturn whiten(GSTLALWhiten *element, GstBuffer *outbuf, guint *out
 				 */
 
 				g_object_notify(G_OBJECT(element), "mean-psd");
-				gst_element_post_message(GST_ELEMENT(element), psd_message_new(element, element->psd));
+				gst_element_post_message(GST_ELEMENT(element), gstlal_whiten_message_psd_new(element, element->instrument, element->psd));
 				if(element->mean_psd_pad) {
 					/* fft_length is sure to be even */
 					GstFlowReturn result = push_psd(element->mean_psd_pad, element->psd, fft_length(element) / 2 - zero_pad_length(element), element->sample_rate);
@@ -794,7 +879,7 @@ static GstFlowReturn whiten(GSTLALWhiten *element, GstBuffer *outbuf, guint *out
 			 */
 
 			if(XLALUnitCompare(&lalDimensionlessUnit, &element->tdworkspace->sampleUnits)) {
-				char units[100];
+				char units[LALUnitTextSize];
 				XLALUnitAsString(units, sizeof(units), &element->tdworkspace->sampleUnits);
 				GST_ERROR_OBJECT(element, "whitening process failed to produce dimensionless time series: result has units \"%s\"", units);
 				return GST_FLOW_ERROR;
@@ -1022,9 +1107,25 @@ static gboolean event(GstBaseTransform *trans, GstEvent *event)
 	switch(GST_EVENT_TYPE(event)) {
 	case GST_EVENT_TAG: {
 		GstTagList *taglist;
+		gchar *instrument;
 		gchar *units;
 
+		/*
+		 * get a pointer to the taglist in the event, then make
+		 * copy.  do not modify the taglist in the event.
+		 */
+
 		gst_event_parse_tag(event, &taglist);
+		taglist = gst_tag_list_copy(taglist);
+
+		if(gst_tag_list_get_string(taglist, GSTLAL_TAG_INSTRUMENT, &instrument)) {
+			/*
+			 * tag list contains an instrument name
+			 */
+			g_free(element->instrument);
+			element->instrument = instrument;
+		}
+
 		if(gst_tag_list_get_string(taglist, GSTLAL_TAG_UNITS, &units)) {
 			/*
 			 * tag list contains a units tag;  replace with
@@ -1037,45 +1138,22 @@ static gboolean event(GstBaseTransform *trans, GstEvent *event)
 			if(!XLALParseUnitString(&sample_units, units)) {
 				GST_ERROR_OBJECT(element, "cannot parse units \"%s\"", units);
 				sample_units = lalDimensionlessUnit;
-				/*
-				 * re-use the event
-				 */
-
-				gst_event_ref(event);
 			} else {
-				gchar dimensionless_units[16];	/* argh hard-coded length = BAD BAD BAD */
+				gchar dimensionless_units[LALUnitTextSize];
 				XLALUnitAsString(dimensionless_units, sizeof(dimensionless_units), &lalDimensionlessUnit);
-
-				/*
-				 * create a new event with a new taglist
-				 * object (don't corrupt the original
-				 * event, 'cause we don't own it)
-				 */
-
-				taglist = gst_tag_list_copy(taglist);
 				/* FIXME:  gstreamer doesn't like empty strings */
 				gst_tag_list_add(taglist, GST_TAG_MERGE_REPLACE, GSTLAL_TAG_UNITS, " "/*dimensionless_units*/, NULL);
-				event = gst_event_new_tag(taglist);
 			}
 
 			g_free(units);
 			element->sample_units = sample_units;
-		} else
-			/*
-			 * re-use the event
-			 */
-
-			gst_event_ref(event);
+		}
 
 		/*
-		 * gst_pad_push_event() consumes the reference count
+		 * gst_element_found_tags() takes ownership of the taglist
 		 */
 
-		if(element->mean_psd_pad) {
-			gst_event_ref(event);
-			gst_pad_push_event(element->mean_psd_pad, event);
-		}
-		gst_pad_push_event(GST_BASE_TRANSFORM_SRC_PAD(trans), event);
+		gst_element_found_tags(GST_ELEMENT(trans), taglist);
 
 		/*
 		 * don't forward the event (we did it)
@@ -1499,6 +1577,8 @@ static void finalize(GObject * object)
 {
 	GSTLALWhiten *element = GSTLAL_WHITEN(object);
 
+	g_free(element->instrument);
+	element->instrument = NULL;
 	if(element->mean_psd_pad) {
 		gst_object_unref(element->mean_psd_pad);
 		element->mean_psd_pad = NULL;
@@ -1678,7 +1758,7 @@ static void gstlal_whiten_class_init(GSTLALWhitenClass *klass)
 		g_param_spec_double(
 			"delta-f",
 			"Delta f",
-			"PSD frequency resolution in Hz",
+			"PSD frequency resolution in Hertz",
 			0, G_MAXDOUBLE, 0,
 			G_PARAM_READABLE | G_PARAM_STATIC_STRINGS
 		)
@@ -1689,7 +1769,7 @@ static void gstlal_whiten_class_init(GSTLALWhitenClass *klass)
 		g_param_spec_double(
 			"f-nyquist",
 			"Nyquist Frequency",
-			"Nyquist frequency in Hz",
+			"Nyquist frequency in Hertz",
 			0, G_MAXDOUBLE, 0,
 			G_PARAM_READABLE | G_PARAM_STATIC_STRINGS
 		)
@@ -1705,7 +1785,7 @@ static void gstlal_whiten_class_init(GSTLALWhitenClass *klass)
 				"bin",
 				"Bin",
 				"Power spectral density bin",
-				-G_MAXDOUBLE, G_MAXDOUBLE, 1.0,
+				0, G_MAXDOUBLE, 1.0,
 				G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS
 			),
 			G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS
@@ -1768,6 +1848,7 @@ static void gstlal_whiten_init(GSTLALWhiten *element, GSTLALWhitenClass *klass)
 
 	element->sample_units = lalDimensionlessUnit;
 	element->sample_rate = 0;
+	element->instrument = NULL;
 	element->input_queue = NULL;
 
 	element->hann_window = NULL;
