@@ -28,11 +28,23 @@
 #
 # A file that contains the datasource module code
 #
+# ### Review Status
+#
+# | Names                                       | Hash                                        | Date       |
+# | ------------------------------------------- | ------------------------------------------- | ---------- |
+# | Florent, Sathya, Duncan Me., Jolien, Kipp, Chad | b3ef077fe87b597578000f140e4aa780f3a227aa    | 2014-05-01 |
+#
+# #### Action items
+#
+# - State vector, DQ, etc must be in place by November and should be coordinated with JRPC/DAC/DetChar/CBC/etc
+# - Consider a dynamic data time out that seneses when data is not going to arrive for a while
 
 ##
 # @package python.datasource
 #
 # datasource module
+#
+
 
 import sys
 import optparse
@@ -67,8 +79,7 @@ lsctables.use_in(ContentHandler)
 # Misc useful functions
 #
 
-## #### A dictionary of channel names from a list
-#  _Python doc string_:
+
 def channel_dict_from_channel_list(channel_list):
 	"""!
 	Given a list of channels, produce a dictionary keyed by ifo of channel names:
@@ -84,8 +95,6 @@ def channel_dict_from_channel_list(channel_list):
 	return dict(instrument_channel.split("=") for instrument_channel in channel_list)
 
 
-## #### A string of channel names from a dictionary
-#  _Python doc string_:
 def pipeline_channel_list_from_channel_dict(channel_dict, ifos = None, opt = "channel-name"):
 	"""!
 	Creates a string of channel names options from a dictionary keyed by ifos.
@@ -131,8 +140,6 @@ state_vector_on_off_dict = {
 }
 
 
-## #### A dictionary of state vector bits from a list
-#  _Python doc string_:
 def state_vector_on_off_dict_from_bit_lists(on_bit_list, off_bit_list, state_vector_on_off_dict = state_vector_on_off_dict):
 	"""!
 	Produce a dictionary (keyed by detector) of on / off bit tuples from a
@@ -182,8 +189,6 @@ def state_vector_on_off_dict_from_bit_lists(on_bit_list, off_bit_list, state_vec
 	return state_vector_on_off_dict
 
 
-## #### A list of state vector command line arguments from a dictionary
-#  _Python doc string_:
 def state_vector_on_off_list_from_bits_dict(bit_dict):
 	"""!
 	Produce a tuple of useful command lines from a dictionary of on / off state
@@ -213,7 +218,7 @@ def state_vector_on_off_list_from_bits_dict(bit_dict):
 	return onstr, offstr
 
 
-## #### framexmit ports in use on the LDG
+## framexmit ports in use on the LDG
 # Look-up table to map instrument name to framexmit multicast address and
 # port
 #
@@ -230,8 +235,34 @@ framexmit_ports = {
 	}
 }
 
-## #### Main organizational data class of this module
-#  _Python doc string_:
+
+def seek_event_for_gps(gps_start_time, gps_end_time, flags = 0):
+	"""!
+	Create a new seek event, i.e., gst.event_new_seek()  for a given
+	gps_start_time and gps_end_time, with optional flags.  
+
+	@param gps_start_time start time as LIGOTimeGPS, double or float
+	@param gps_end_time start time as LIGOTimeGPS, double or float
+	"""
+	def seek_args_for_gps(gps_time):
+		"""!
+		Convenience routine to convert a GPS time to a seek type and a
+		GStreamer timestamp.
+		"""
+
+		if gps_time is None or gps_time == -1:
+			return (gst.SEEK_TYPE_NONE, -1) # -1 == gst.CLOCK_TIME_NONE
+		elif hasattr(gps_time, 'ns'):
+			return (gst.SEEK_TYPE_SET, gps_time.ns())
+		else:
+			return (gst.SEEK_TYPE_SET, long(float(gps_time) * gst.SECOND))
+
+	start_type, start_time = seek_args_for_gps(gps_start_time)
+	stop_type, stop_time   = seek_args_for_gps(gps_end_time)
+
+	return gst.event_new_seek(1., gst.FORMAT_TIME, flags, start_type, start_time, stop_type, stop_time)
+
+
 class GWDataSourceInfo(object):
 	"""!
 	Hold the data associated with data source command lines.
@@ -243,29 +274,23 @@ class GWDataSourceInfo(object):
 		""" 
 
 		## A list of possible, valid data sources ("frames", "framexmit", "lvshm", "nds", "white", "silence", "AdvVirgo", "LIGO", "AdvLIGO")
-		self.data_sources = ("frames", "framexmit", "lvshm", "nds", "white", "silence", "AdvVirgo", "LIGO", "AdvLIGO")
-
-		## Callback function to handle the "start" signals from the gate
-		self.gate_start_callback = None
-
-		## Callback function to handle the "stop" signals from the gate
-		self.gate_stop_callback = None
+		self.data_sources = set(("frames", "framexmit", "lvshm", "nds", "white", "silence", "AdvVirgo", "LIGO", "AdvLIGO"))
+		self.live_sources = set(("framexmit", "lvshm"))
+		assert self.live_sources <= self.data_sources
 
 		# Sanity check the options
 		if options.data_source not in self.data_sources:
 			raise ValueError("--data-source must be one of %s" % ", ".join(self.data_sources))
 		if options.data_source == "frames" and options.frame_cache is None:
 			raise ValueError("--frame-cache must be specified when using --data-source=frames")
-		if (options.gps_start_time is None or options.gps_end_time is None) and options.data_source == "frames":
-			raise ValueError("--gps-start-time and --gps-end-time must be specified unless --data-source=lvshm")
-		if len(options.channel_name) == 0:
+		if not options.channel_name:
 			raise ValueError("must specify at least one channel in the form --channel-name=IFO=CHANNEL-NAME")
 		if options.frame_segments_file is not None and options.data_source != "frames":
-			raise ValueError("Can only give --frame-segments-file if --data-source=frames")
+			raise ValueError("can only give --frame-segments-file if --data-source=frames")
 		if options.frame_segments_name is not None and options.frame_segments_file is None:
-			raise ValueError("Can only specify --frame-segments-name if --frame-segments-file is given")
+			raise ValueError("can only specify --frame-segments-name if --frame-segments-file is given")
 		if options.data_source == "nds" and (options.nds_host is None or options.nds_port is None):
-			raise ValueError("Must specify --nds-host and --nds-port when using --data-source=nds")
+			raise ValueError("must specify --nds-host and --nds-port when using --data-source=nds")
 
 		## A dictionary of the requested channels, e.g., {"H1":"LDAS-STRAIN", "L1":"LDAS-STRAIN"}
 		self.channel_dict = channel_dict_from_channel_list(options.channel_name)
@@ -282,16 +307,36 @@ class GWDataSourceInfo(object):
 		self.seg = None
 
 		if options.gps_start_time is not None:
+			if options.gps_end_time is None:
+				raise ValueError("must provide both --gps-start-time and --gps-end-time")
+			if options.data_source in self.live_sources:
+				raise ValueError("cannot set --gps-start-time or --gps-end-time with %s" % " or ".join("--data-source=%s" % src for src in sorted(self.live_sources)))
+			try:
+				start = LIGOTimeGPS(options.gps_start_time)
+			except ValueError:
+				raise ValueError("invalid --gps-start-time '%s'" % options.gps_start_time)
+			try:
+				end = LIGOTimeGPS(options.gps_end_time)
+			except ValueError:
+				raise ValueError("invalid --gps-end-time '%s'" % options.gps_end_time)
+			if start >= end:
+				raise ValueError("--gps-start-time must be < --gps-end-time: %s < %s" % (options.gps_start_time, options.gps_end_time))
 			## Segment from gps start and stop time if given
 			self.seg = segments.segment(LIGOTimeGPS(options.gps_start_time), LIGOTimeGPS(options.gps_end_time))
 			## Seek event from the gps start and stop time if given
 			self.seekevent = gst.event_new_seek(1., gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_KEY_UNIT, gst.SEEK_TYPE_SET, self.seg[0].ns(), gst.SEEK_TYPE_SET, self.seg[1].ns())
+		elif options.gps_end_time is not None:
+			raise ValueError("must provide both --gps-start-time and --gps-end-time")
+		elif options.data_source not in self.live_sources:
+			raise ValueError("--gps-start-time and --gps-end-time must be specified when %s" % " or ".join("--data-source=%s" % src for src in sorted(self.live_sources)))
 
 		if options.frame_segments_file is not None:
 			## Frame segments from a user defined file
 			self.frame_segments = ligolw_segments.segmenttable_get_by_name(utils.load_filename(options.frame_segments_file, contenthandler=ContentHandler), options.frame_segments_name).coalesce()
 			if self.seg is not None:
-				## Frame segments will be clipped to seek segment if it exists
+				# Clip frame segments to seek segment if it
+				# exists (not required, just saves some
+				# memory and I/O overhead)
 				self.frame_segments = segments.segmentlistdict((instrument, seglist & segments.segmentlist([self.seg])) for instrument, seglist in self.frame_segments.items())
 		else:
 			## if no frame segments provided, set them to an empty segment list dictionary
@@ -331,8 +376,7 @@ class GWDataSourceInfo(object):
 			## Store the ndssrc specific options: channel_type
 			self.nds_channel_type = options.nds_channel_type
 
-## #### Generic data source options used by many programs to append options to an OptionParser
-# _Python doc string_:
+
 def append_options(parser):
 	"""!
 	Append generic data source options to an OptionParser object in order
@@ -402,7 +446,7 @@ def append_options(parser):
 -	--state-vector-off-bits [hex]
 		Set the state vector off bits to process (optional).
 		The default is 0x160 for all detectors. Override with IFO=bits can be given multiple times.
-		Only currently has meaning for online (lvshm) data.)
+		Only currently has meaning for online (lvshm, framexmit) data
 
 	#### Typical usage case examples
 
@@ -452,9 +496,8 @@ def do_seek(pipeline, seekevent):
 ## @endcond
 
 
-## #### Gate controlled by a segment source
-#
-# ##### Gstreamer graph describing this function:
+##
+# _Gstreamer graph describing this function:_
 #
 # @dot
 # digraph G {
@@ -470,22 +513,23 @@ def do_seek(pipeline, seekevent):
 #	lal_segmentsrc -> lal_gate;
 # }
 # @enddot
-# _Python doc string_:
-def mksegmentsrcgate(pipeline, src, segment_list, seekevent = None, invert_output = False):
+#
+#
+def mksegmentsrcgate(pipeline, src, segment_list, seekevent = None, invert_output = False, **kwargs):
 	"""!
 	Takes a segment list and produces a gate driven by it. Hook up your own input and output.
-	"""
 
-	segsrc = pipeparts.mksegmentsrc(pipeline, segment_list, invert_output=invert_output)
+	@param kwargs passed through to pipeparts.mkgate(), e.g., used to set the gate's name.
+	"""
+	segsrc = pipeparts.mksegmentsrc(pipeline, segment_list, invert_output = invert_output)
 	# FIXME:  remove
 	if seekevent is not None:
 		do_seek(pipeline, seekevent)
-	return pipeparts.mkgate(pipeline, src, threshold = 1, control = segsrc)
+	return pipeparts.mkgate(pipeline, src, threshold = 1, control = segsrc, **kwargs)
 
 
-## #### All-in-one data source
-#
-# ##### Gstreamer graph describing this function
+##
+# _Gstreamer graph describing this function:_
 #
 # @dot
 # digraph mkbasicsrc {
@@ -542,7 +586,8 @@ def mksegmentsrcgate(pipeline, src, segment_list, seekevent = None, invert_outpu
 #	audioconv -> progress -> sim -> queue -> "?"
 # }
 # @enddot
-# _Python doc string_:
+#
+#
 def mkbasicsrc(pipeline, gw_data_source_info, instrument, verbose = False):
 	"""!
 	All the conditionals and stupid pet tricks for reading real or
@@ -570,9 +615,9 @@ def mkbasicsrc(pipeline, gw_data_source_info, instrument, verbose = False):
 	elif gw_data_source_info.data_source == "frames":
 		if instrument == "V1":
 			#FIXME Hack because virgo often just uses "V" in the file names rather than "V1".  We need to sieve on "V"
-			src = pipeparts.mklalcachesrc(pipeline, location = gw_data_source_info.frame_cache, use_mmap = True, cache_src_regex = "V")
+			src = pipeparts.mklalcachesrc(pipeline, location = gw_data_source_info.frame_cache, cache_src_regex = "V")
 		else:
-			src = pipeparts.mklalcachesrc(pipeline, location = gw_data_source_info.frame_cache, use_mmap = True, cache_src_regex = instrument[0], cache_dsc_regex = instrument)
+			src = pipeparts.mklalcachesrc(pipeline, location = gw_data_source_info.frame_cache, cache_src_regex = instrument[0], cache_dsc_regex = instrument)
 		demux = pipeparts.mkframecppchanneldemux(pipeline, src, do_file_checksum = True, channel_list = map("%s:%s".__mod__, gw_data_source_info.channel_dict.items()))
 		pipeparts.framecpp_channeldemux_set_units(demux, dict.fromkeys(demux.get_property("channel-list"), "strain"))
 		# allow frame reading and decoding to occur in a diffrent
@@ -582,7 +627,7 @@ def mkbasicsrc(pipeline, gw_data_source_info, instrument, verbose = False):
 		if gw_data_source_info.frame_segments[instrument] is not None:
 			# FIXME:  make segmentsrc generate segment samples at the sample rate of h(t)?
 			# FIXME:  make gate leaky when I'm certain that will work.
-			src = pipeparts.mkgate(pipeline, src, threshold = 1, control = pipeparts.mksegmentsrc(pipeline, gw_data_source_info.frame_segments[instrument]))
+			src = pipeparts.mkgate(pipeline, src, threshold = 1, control = pipeparts.mksegmentsrc(pipeline, gw_data_source_info.frame_segments[instrument]), name = "%s_frame_segments_gate" % instrument)
 			pipeparts.framecpp_channeldemux_check_segments.set_probe(src.get_pad("src"), gw_data_source_info.frame_segments[instrument])
 		# FIXME:  remove this when pipeline can handle disconts
 		src = pipeparts.mkaudiorate(pipeline, src, skip_to_first = True, silent = False)
@@ -625,13 +670,8 @@ def mkbasicsrc(pipeline, gw_data_source_info, instrument, verbose = False):
 
 		# use state vector to gate strain
 		src = pipeparts.mkgate(pipeline, strain, threshold = 1, control = statevector, default_state = False, name = "%s_state_vector_gate" % instrument)
-		# export state vector state
-		if gw_data_source_info.gate_start_callback is not None:
-			src.set_property("emit-signals", True)
-			src.connect("start", gw_data_source_info.gate_start_callback)
-		if gw_data_source_info.gate_stop_callback is not None:
-			src.set_property("emit-signals", True)
-			src.connect("stop", gw_data_source_info.gate_stop_callback)
+
+		# fill in holes, skip duplicate data
 		src = pipeparts.mkaudiorate(pipeline, src, skip_to_first = True, silent = False)
 		@bottle.route("/%s/strain_add_drop.txt" % instrument)
 		def strain_add(elem = src):
@@ -689,9 +729,8 @@ def mkbasicsrc(pipeline, gw_data_source_info, instrument, verbose = False):
 	return src
 
 
-## #### h(t) Gate: quick glitch excision trick
-#
-# ##### Gstreamer graph
+## 
+# _Gstreamer graph describing the pipeline_
 #
 # @dot
 # digraph G {
@@ -700,32 +739,30 @@ def mkbasicsrc(pipeline, gw_data_source_info, instrument, verbose = False):
 #	rankdir=LR;
 # 	tee [URL="\ref pipeparts.mktee()"];
 # 	inputqueue [URL="\ref pipeparts.mkqueue()"];
-# 	controlqueue [URL="\ref pipeparts.mkqueue()"];
 #	lal_gate [URL="\ref pipeparts.mkgate()"];
 #	in [label="?"];
 #	out [label="?"];
 #	in -> tee -> inputqueue -> lal_gate -> out;
-#	tee -> controlqueue -> lal_gate;
+#	tee -> lal_gate;
 # }
 # @enddot
-# _Python doc string_:
-def mkhtgate(pipeline, src, control = None, threshold = 8.0, attack_length = -128, hold_length = -128, name = None):
+#
+#
+def mkhtgate(pipeline, src, control = None, threshold = 8.0, attack_length = 128, hold_length = 128, **kwargs):
 	"""!
 	A convenience function to provide thresholds on input data.  This can
 	be used to remove large spikes / glitches etc.  Of course you can use it for
 	other stuff by plugging whatever you want as input and ouput
-	"""
 
+	NOTE:  the queues constructed by this code assume the attack and
+	hold lengths combined are less than 1 second in duration.
+	"""
 	# FIXME someday explore a good bandpass filter
 	# src = pipeparts.mkaudiochebband(pipeline, src, low_frequency, high_frequency)
-	src = pipeparts.mktee(pipeline, src)
 	if control is None:
-		control = pipeparts.mkqueue(pipeline, src, max_size_time = 0, max_size_bytes = 0, max_size_buffers = 0)
-	input = pipeparts.mkqueue(pipeline, src, max_size_time = gst.SECOND, max_size_bytes = 0, max_size_buffers = 0)
-	if name is not None:
-		return pipeparts.mkgate(pipeline, input, threshold = threshold, control = control, attack_length = attack_length, hold_length = hold_length, invert_control = True, name = name)
-	else:
-		return pipeparts.mkgate(pipeline, input, threshold = threshold, control = control, attack_length = attack_length, hold_length = hold_length, invert_control = True)
+		control = src = pipeparts.mktee(pipeline, src)
+	src = pipeparts.mkqueue(pipeline, src, max_size_time = gst.SECOND, max_size_bytes = 0, max_size_buffers = 0)
+	return pipeparts.mkgate(pipeline, src, control = control, threshold = threshold, attack_length = -attack_length, hold_length = -hold_length, invert_control = True, **kwargs)
 
 # Unit tests
 if __name__ == "__main__":
