@@ -100,14 +100,30 @@ def mkgeneric(pipeline, src, elem_type_name, **properties):
 
 
 class src_deferred_link(object):
-	def __init__(self, src, srcpadname, sinkpad):
-		no_more_pads_handler_id = src.connect("no-more-pads", self.no_more_pads, srcpadname)
-		src.connect("pad-added", self.pad_added, (srcpadname, sinkpad, no_more_pads_handler_id))
+	"""!
+	A class that manages the task of watching for and connecting to new
+	source pads by name.  The inputs are an element, the name of the
+	source pad to watch for on that element, and the sink pad (on a
+	different element) to which the source pad should be linked when it
+	appears.
+
+	The "pad-added" signal of the element will be used to watch for new
+	pads, and if the "no-more-pads" signal is emitted by the element
+	before the requested pad has appeared ValueException is raised.
+	"""
+	def __init__(self, element, srcpadname, sinkpad):
+		no_more_pads_handler_id = element.connect("no-more-pads", self.no_more_pads, srcpadname)
+		assert no_more_pads_handler_id > 0
+		pad_added_data = [srcpadname, sinkpad, no_more_pads_handler_id]
+		pad_added_handler_id = element.connect("pad-added", self.pad_added, pad_added_data)
+		assert pad_added_handler_id > 0
+		pad_added_data.append(pad_added_handler_id)
 
 	@staticmethod
-	def pad_added(element, pad, (srcpadname, sinkpad, no_more_pads_handler_id)):
+	def pad_added(element, pad, (srcpadname, sinkpad, no_more_pads_handler_id, pad_added_handler_id)):
 		if pad.get_name() == srcpadname:
-			pad.get_parent().handler_disconnect(no_more_pads_handler_id)
+			element.handler_disconnect(no_more_pads_handler_id)
+			element.handler_disconnect(pad_added_handler_id)
 			pad.link(sinkpad)
 
 	@staticmethod
@@ -266,8 +282,8 @@ def mksegmentsrc(pipeline, segment_list, blocksize = 4096 * 1 * 1, invert_output
 
 
 ## Adds a <a href="@gstlalgtkdoc/GstLALCacheSrc.html">lal_cachesrc</a> element to a pipeline with useful default properties
-def mklalcachesrc(pipeline, location, **properties):
-	return mkgeneric(pipeline, None, "lal_cachesrc", location = location, **properties)
+def mklalcachesrc(pipeline, location, use_mmap = True, **properties):
+	return mkgeneric(pipeline, None, "lal_cachesrc", location = location, use_mmap = use_mmap, **properties)
 
 
 def mklvshmsrc(pipeline, shm_name, **properties):
@@ -352,9 +368,11 @@ def mkaudiotestsrc(pipeline, **properties):
 
 
 ## see documentation for mktaginject() mkcapsfilter() and mkaudiotestsrc()
-def mkfakesrc(pipeline, instrument, channel_name, blocksize = 16384 * 8 * 1, volume = 1e-20, is_live = False, wave = 9, rate = 16384):
-	# default blocksize is 1 second of double precision floats at
-	# 16384 Hz, e.g., h(t)
+def mkfakesrc(pipeline, instrument, channel_name, blocksize = None, volume = 1e-20, is_live = False, wave = 9, rate = 16384):
+	if blocksize is None:
+		# default blocksize is 1 second * rate samples/second * 8
+		# bytes/sample (assume double-precision floats)
+		blocksize = 1 * rate * 8
 	return mktaginject(pipeline, mkcapsfilter(pipeline, mkaudiotestsrc(pipeline, samplesperbuffer = blocksize / 8, wave = wave, volume = volume, is_live = is_live), "audio/x-raw-float, width=64, rate=%d" % rate), "instrument=%s,channel-name=%s,units=strain" % (instrument, channel_name))
 
 
@@ -442,6 +460,15 @@ def mktee(pipeline, src):
 	return mkgeneric(pipeline, src, "tee")
 
 
+## Adds a <a href="@gstdoc/GstLALAdder.html">lal_adder</a> element to a pipeline with useful default properties
+def mkadder(pipeline, srcs, sync = True, **properties):
+	elem = mkgeneric(pipeline, None, "lal_adder", sync = sync, **properties)
+	if srcs is not None:
+		for src in srcs:
+			src.link(elem)
+	return elem
+
+
 ## Adds a <a href="@gstdoc/gstreamer-plugins-queue.html">queue</a> element to a pipeline with useful default properties
 def mkqueue(pipeline, src, **properties):
 	return mkgeneric(pipeline, src, "queue", **properties)
@@ -474,6 +501,23 @@ def mkiirbank(pipeline, src, a1, b0, delay, name=None):
 	return elem
 
 
+def mkcudaiirbank(pipeline, src, a1, b0, delay, name=None):
+ 	properties = dict((name, value) for name, value in (("name", name), ("delay_matrix", delay)) if value is not None)
+ 	if a1 is not None:
+ 		properties["a1_matrix"] = pipeio.repack_complex_array_to_real(a1)
+ 	if b0 is not None:
+ 		properties["b0_matrix"] = pipeio.repack_complex_array_to_real(b0)
+ 	elem = mkgeneric(pipeline, src, "cuda_iirbank", **properties)
+ 	elem = mknofakedisconts(pipeline, elem)	# FIXME:  remove after basetransform behaviour fixed
+ 	return elem
+
+
+def mkcudamultiratespiir(pipeline, src, bank_struct, bank_id=0, name=None):
+	properties = dict((name, value) for name, value in (("name", name), ("spiir_bank", bank_struct), ("bank_id", bank_id)) if value is not None)
+	elem = mkgeneric(pipeline, src, "cuda_multiratespiir", **properties)
+	return elem
+
+
 def mktrim(pipeline, src, initial_offset = None, final_offset = None, inverse = None):
 	properties = dict((name, value) for name, value in zip(("initial-offset", "final-offset", "inverse"), (initial_offset,final_offset,inverse)) if value is not None)
 	return mkgeneric(pipeline, src, "lal_trim", **properties)
@@ -824,9 +868,17 @@ def mklhocoherentnull(pipeline, H1src, H2src, H1_impulse, H1_latency, H2_impulse
 			peer.link_pads(None, elem, padname)
 	return elem
 
+def mkcomputegamma(pipeline, dctrl, exc, cos, sin, **properties):
+	elem = mkgeneric(pipeline, None, "lal_compute_gamma", **properties)
+	for peer, padname in ((dctrl, "dctrl_sink"), (exc, "exc_sink"), (cos, "cos"), (sin, "sin")):
+		if isinstance(peer, gst.Pad):
+			peer.get_parent_element().link_pads(peer, elem, padname)
+		elif peer is not None:
+			peer.link_pads(None, elem, padname)
+	return elem
 
-def mkbursttriggergen(pipeline, src, n, bank):
-	return mkgeneric(pipeline, src, "lal_bursttriggergen", n = n, bank_filename = bank)
+def mkbursttriggergen(pipeline, src, **properties):
+	return mkgeneric(pipeline, src, "lal_bursttriggergen", **properties)
 
 def mkodctodqv(pipeline, src, **properties):
 	return mkgeneric(pipeline, src, "lal_odc_to_dqv", **properties)
