@@ -25,6 +25,7 @@
 
 
 import sys
+import os
 
 
 import pygtk
@@ -39,9 +40,23 @@ import signal
 
 ## @file
 # The simplehander module
+#
+# ### Review Status
+#
+# | Names                                       | Hash                                        | Date       |
+# | ------------------------------------------- | ------------------------------------------- | ---------- |
+# | Florent, Sathya, Duncan Me., Jolien, Kipp, Chad | b3ef077fe87b597578000f140e4aa780f3a227aa    | 2014-05-01 |
+#
+# #### Action items
+#
+# - Get event based programming expert to have a look?
+#
 
 ## @package python.simplehandler
 # The simplehander module
+#
+
+
 
 #
 # =============================================================================
@@ -66,7 +81,35 @@ class Handler(object):
 
 		bus = pipeline.get_bus()
 		bus.add_signal_watch()
-		bus.connect("message", self.on_message)
+		self.on_message_handler_id = bus.connect("message", self.on_message)
+
+		def excepthook(*args):
+			# system exception hook that forces hard exit.  without this,
+			# exceptions that occur inside python code invoked as a call-back
+			# from the gstreamer pipeline just stop the pipeline, they don't
+			# cause gstreamer to exit.
+
+			# FIXME:  they probably *would* cause if we could figure out why
+			# element errors and the like simply stop the pipeline instead of
+			# crashing it, as well.  Perhaps this should be removed when/if the
+			# "element error's don't crash program" problem is fixed
+			sys.__excepthook__(*args)
+			os._exit(1)
+
+		sys.excepthook = excepthook
+
+
+	def quit(self, bus):
+		"""
+		Decouple this object from the Bus object to allow the Bus'
+		reference count to drop to 0, and .quit() the mainloop
+		object.  This method is invoked by the default EOS and
+		ERROR message handlers.
+		"""
+		bus.disconnect(self.on_message_handler_id)
+		del self.on_message_handler_id
+		bus.remove_signal_watch()
+		self.mainloop.quit()
 
 	def do_on_message(self, bus, message):
 		"""!
@@ -83,7 +126,7 @@ class Handler(object):
 			pass
 		elif message.type == gst.MESSAGE_EOS:
 			self.pipeline.set_state(gst.STATE_NULL)
-			self.mainloop.quit()
+			self.quit(bus)
 		elif message.type == gst.MESSAGE_INFO:
 			gerr, dbgmsg = message.parse_info()
 			print >>sys.stderr, "info (%s:%d '%s'): %s" % (gerr.domain, gerr.code, gerr.message, dbgmsg)
@@ -94,7 +137,7 @@ class Handler(object):
 			gerr, dbgmsg = message.parse_error()
 			# FIXME:  this deadlocks.  shouldn't we be doing this?
 			#self.pipeline.set_state(gst.STATE_NULL)
-			self.mainloop.quit()
+			self.quit(bus)
 			sys.exit("error (%s:%d '%s'): %s" % (gerr.domain, gerr.code, gerr.message, dbgmsg))
 
 
